var keypad = {
	keys: [
		{text: "A", value: 65},
		{text: "B", value: 66},
		{text: "C", value: 67},
		{text: "D", value: 68},
		{text: "E", value: 69},
		{text: "F", value: 70},
		{text: "G", value: 71},
		{text: "H", value: 72},
		{text: "I", value: 73},
		{text: "J", value: 74},
		{text: "K", value: 75},
		{text: "L", value: 76},
		{text: "M", value: 77},
		{text: "N", value: 78},
		{text: "O", value: 79},
		{text: "P", value: 80},
		{text: "Q", value: 81},
		{text: "R", value: 82},
		{text: "S", value: 83},
		{text: "T", value: 84},
		{text: "U", value: 85},
		{text: "V", value: 86},
		{text: "W", value: 87},
		{text: "X", value: 88},
		{text: "Y", value: 89},
		{text: "Z", value: 90},
		{text: "&larr;", value: 8},
	],
	init: function() {
		this.keys.forEach(function(key) {
			var button = $("<div class='key' data-value='" + key.value + "'>" + key.text + "</div>");
			button.bind('click', function() {
				var keyDownEvent = $.Event("keydown");
				keyDownEvent.which = parseInt($(this).attr('data-value'));				
				$(document).trigger(keyDownEvent);
			});

			button.bind('touchstart', function() {
				var popup = $(this).clone().css({'position': 'fixed', 'top': $(this).offset().top-$(this).height()-3, 'left': $(this).offset().left-3});
				popup.addClass('popup');
				var popuphtml = popup.wrap('<p/>').parent().html();
				$(this).append(popuphtml);
				setTimeout(function() { 
					$(".popup").remove(); 
				}, 250);
			});

			$("#keyboard").append(button);
		});
	}
};

var qBoard = {
	direction: "across",
	init: function(hasSavedState) {
		$('.dirtab').bind('click', function() {
			if($(this).attr("id") === "acrosstab") qBoard.reset("across");
			else qBoard.reset("down");
			$('.selqkey').click();
		});
		if(hasSavedState) state.setCurrentQuestion();
		else $("#acrosstab").click();
	},	
	reset: function(dir, qno) {
		this.direction = dir;
		$('.seldirtab').removeClass('seldirtab');
		if(dir === "across") $("#acrosstab").addClass('seldirtab');
		else $("#downtab").addClass('seldirtab');
		qBoard.renderQuestionKeys(qno);
	},
	renderQuestionKeys: function(selqno) {
		$("#qlist").empty();
		var questionSet = this.direction === "across" ? questions.across : questions.down;
		for(var qno in questionSet) {
			if(!questionSet.hasOwnProperty(qno)) continue;
			if(selqno === undefined) selqno = qno;
			$("#qlist").append("<div class='qkey'>" + qno + "</div>");
		}
		this.selectQuestion(selqno);
		this.setQuestionKeyWidth();

		$('.qkey').bind('click', function() {
			qBoard.selectQuestion($(this).text());
			var qStartGrid = util.findQStartGrid($(this).text());
			if(!state.equalsCurrGrid(qStartGrid)) {
				state.direction = qBoard.direction;				
			} else if(state.direction === qBoard.direction) return;
			qStartGrid.click();
		});
	},
	selectQuestion: function(qno) {
			$('.selqkey').removeClass('selqkey');
			$('.qkey').filter(function() {
				return $(this).text() == qno;
			}).addClass('selqkey');
	},
	setQuestionKeyWidth: function() {
		var numQuestions = $('.qkey').length;
		var questionSectionSize = $('#qlist').innerWidth();
		var qkeyWidth =  ((questionSectionSize/numQuestions) - 3) + 'px';
		$('.qkey').css({'width': qkeyWidth});
	}
};

var util = {
	isQuestionGrid: function(row, column) {
		if(!crosswordGrid[row][column]) return false;
		if((row === 0 || !crosswordGrid[row-1][column]) && (row !== crosswordGrid.length-1 && crosswordGrid[row+1][column])) 
			return true; 
		if((column === 0 || !crosswordGrid[row][column-1]) && (column !== crosswordGrid[row].length-1 && crosswordGrid[row][column+1]))
			return true;
		return false; 
	},
	findGrid: function(x, y) {
		return $("div[data-x=" + x + "][data-y=" + y + "].white");
	},
	findQStartGrid: function(qno) {
		return $("div[data-qno=" + qno + "].white");
	},
	resizeElements: function() {
		var widthPercent = 100;
		var gridNumberPercent = 30;
		var gridContentPercent = 50;
		var menuButtonContentPercent = 75;
		var questionsHeightPercent = 8*gridContentPercent;
		var arrowSizePercent = 15;
		$('.grid').css({'width': util.percentGridWidth(widthPercent), 'height': util.percentGridWidth(widthPercent)});
		$('.gridnumber').css({'font-size': this.percentGridWidth(gridNumberPercent), 'height': this.percentGridWidth(gridNumberPercent)});
		$('.gridcontent').css({'font-size': this.percentGridWidth(gridContentPercent), 'line-height': this.percentGridWidth(widthPercent)});
		$('.acrossarrowbox').css({'font-size': this.percentGridWidth(arrowSizePercent), 'line-height': this.percentGridWidth(widthPercent)});
		$('.downarrowbox').css({'font-size': this.percentGridWidth(arrowSizePercent)});
		$('#questions').css({'font-size': this.percentGridWidth(gridContentPercent), 'width': this.getViewPortSize()-1, 'height': this.percentGridWidth(questionsHeightPercent)});
		$('#menu').css({'width': this.percentGridWidth(menuButtonContentPercent), 'height': this.percentGridWidth(menuButtonContentPercent)});
		$('#keyboard').css({'font-size': this.percentGridWidth(gridContentPercent), 'width': this.getViewPortSize()});
		qBoard.setQuestionKeyWidth();
		var keyFontPercent = 40;
		$('.key').css({'width': this.percentKeyWidth(widthPercent), 'height': this.percentKeyWidth(widthPercent), 'margin': this.getKeyMargin(), 'line-height': this.percentKeyWidth(widthPercent), 'font-size': this.percentKeyWidth(keyFontPercent)});
		this.centerAnswerGrid();
	},
	percentGridWidth: function(percent) {
		var gridWidth = (this.getViewPortSize() - 15)/15;
		return gridWidth*(percent/100)+'px';
	},
	percentKeyWidth: function(percent) {
		return this.getKeyWidth()*(percent/100)+'px';
	},
	getKeyWidth: function() {
		return this.getViewPortSize()*(3/4)/9;
	},
	getKeyMargin: function() {
		var keyMargin = (this.getViewPortSize() - (this.getKeyWidth()*9))/23;
		return keyMargin+'px';
	},
	getViewPortSize: function() {
		return $('#answergrid').innerWidth()
	},
	centerAnswerGrid: function() {
		var availableHeight = $('#keyboard').offset().top - ($('#questions').offset().top + $('#questions').height());
		var marginTop = ((availableHeight - this.getViewPortSize())/2) + 'px';
		$('#answergrid').css({'margin-top': marginTop});
	},
	gridClickHandler: function(event){
		state.resetCurrGrid();
		if($(this).attr("class").indexOf("black") >= 0) return;
		$(this).addClass("selected");
		if(state.equalsCurrGrid($(this))) state.switchDirection();
		state.currGrid = $(this);
		if(!state.isDirectionValid()) state.switchDirection();
		state.setArrows();
		state.setCurrentQuestion();
		state.save();
	},
	saveQuestions: function() {
		localStorage.questions = questions;
	}
};

var menu = {
	dialogBox: null,
	setupDialogBox: function() {
		this.dialogBox = $("#menu-dialog").dialog({		
			autoOpen: false,
			height: 110,
			width: 200,
			modal: true,
			close: function() {
			}
		});
		$(".ui-dialog-titlebar").hide();
	},
	init: function() {
		this.setupDialogBox();
		this.wireMenuButton();
		this.wireResetButton();
		this.wireCancelButton();
	},
	wireMenuButton: function() {
		$("#menu").bind('click', function() {
			menu.dialogBox.dialog("open");
		});
	},
	wireResetButton: function() {
		$('#reset').bind('click', function() {
			state.resetToLatest();
		});
	},
	wireCancelButton: function() {
		$("#cancelMenu").bind('click', function() {
			menu.dialogBox.dialog("close");
		});
	}
};

var state = {
	currGrid: null,
	resetCurrGrid: function() {
		if(this.currGrid !== null) {
			this.currGrid.removeClass("selected");
			this.resetArrows();
		}
	},
	equalsCurrGrid: function(otherGrid) {
		return this.currGrid !== null && this.currGrid.attr('data-x') == otherGrid.attr('data-x') && this.currGrid.attr('data-y') == otherGrid.attr('data-y');
	},
	direction: "across",
	setArrows: function() {
		this.resetArrows();
		if(this.direction === "across") this.currGrid.find('.acrossarrowbox').html("&#9660;");
		else this.currGrid.find('.downarrowbox').html("&#9660;");
	},
	resetArrows: function() {
		this.currGrid.find('.acrossarrowbox').html('');
		this.currGrid.find('.downarrowbox').html('');
	},
	switchDirection: function() {
		if(this.direction === "across")	this.direction = "down";
		else this.direction = "across";
	},
	isDirectionValid: function() {
		var canMoveForward = this.move("forward").length != 0;
		if(canMoveForward) return true;

		var canMoveBackward = this.move("backward").length != 0;
		return canMoveBackward;
	},
	move: function(cursordir, step) {
		if(step === undefined) step = 1;
		var currX = +this.currGrid.attr('data-x');
		var currY = +this.currGrid.attr('data-y');
		var delta = cursordir === "forward" ? step : -step;
		var newX = this.direction === "across" ? currX : currX+delta;
		var newY = this.direction === "across" ? currY+delta : currY;
		return util.findGrid(newX, newY);
	},
	getCurrQuestion: function() {
		var moveSteps = 0;
		var inspectGrid = this.currGrid;
		var qno;
		while(inspectGrid !== "undefined" && inspectGrid.length !== 0) {
			var candidateqno = inspectGrid.attr('data-qno');
			if(candidateqno !== undefined) qno = candidateqno;
			inspectGrid = this.move(moveSteps, ++moveSteps);
		}
		return qno;
	},
	setCurrentQuestion: function() {
		var currqno = this.getCurrQuestion();
		if(this.direction === "across") $("#currq").text(questions.across[currqno]);
		else $("#currq").text(questions.down[currqno]);
		qBoard.reset(this.direction, currqno);
	},
	save: function() {
		if(localStorage.questions === undefined) {
			localStorage.questions = JSON.stringify(questions);
		}

		localStorage.gridState = $("#answergrid").html();
		localStorage.direction = this.direction;
	},
	tryLoadPreviousState: function() {
		if(localStorage.questions !== undefined && localStorage.gridState !== undefined) {
			questions = JSON.parse(localStorage.questions);
			$("#answergrid").html(localStorage.gridState);
			$(".grid").each(function() {
				$(this).bind('click', util.gridClickHandler);
			});
			this.currGrid = $(".selected");
			this.direction = localStorage.direction;
			this.setCurrentQuestion();
			return true;
		}
		return false;
	},
	resetToLatest: function() {
		localStorage.clear();
		location.reload();
	}
};

var questionNumberGenerator = {
	nextNumber: 1,
	getNext: function() {
		return this.nextNumber++;
	}
};

var gridGenerator = {
	createRow: function(el) {
		var row = $("<div class='row'></div>");
		$(el).append(row);
		return row;
	},
	createGrid: function(el, color, row, column) {
		var qno = '';
		if(util.isQuestionGrid(row, column)) qno = questionNumberGenerator.getNext();
		
		var grid = $("<div class='grid'><div class='gridnumber'>" + qno + "</div><div class='gridcontent'></div><div class='downarrowbox'></div><div class='acrossarrowbox'></div></div>");
		grid.addClass(color);
		grid.attr('data-x', row);
		grid.attr('data-y', column);		
		if(qno) grid.attr('data-qno', qno);		
		grid.bind('click', util.gridClickHandler);
		$(el).append(grid);
		return grid;
	}
}

$(document).ready(function(){
	crosswordGrid.forEach(function(row, rindex) {
		var rowEl = gridGenerator.createRow($("#answergrid"));
		row.forEach(function(element, cindex) {
			var color = element ? "white" : "black";
			var grid = gridGenerator.createGrid(rowEl, color, rindex, cindex);
		});
	});

	$(document).keydown(function(keyDownEvent) {
		keyDownEvent.preventDefault();
		var newText, direction;
		var isDeleteKey = keyDownEvent.which === 8 || keyDownEvent.which === 46;
		var newText = isDeleteKey ? "" : String.fromCharCode(keyDownEvent.which).toUpperCase();
		var cursordir = isDeleteKey ? "backward" : "forward";
		state.currGrid.find('.gridcontent').text(newText);
		var next = state.move(cursordir);
		if(next.length !== 0) next.click();
		state.save();
	});

	var hasSavedState = state.tryLoadPreviousState();
	if(!hasSavedState) {
		state.save();
	}

	qBoard.init(hasSavedState);
	menu.init();
	keypad.init();
	util.resizeElements();
});

$(window).resize(function() {
	util.resizeElements();
});